@@ -1,6 +1,6 @@
 package Net::SFTP::Foreign::Backend::Unix;
 
-our $VERSION = '1.74_01';
+our $VERSION = '1.88_02';
 
 use strict;
 use warnings;
@@ -150,13 +150,13 @@ sub _init_transport {
                           delete($opts->{asks_for_username_at_login}) );
                 if ($ask_for_username_at_login) {
                     croak "ask_for_username_at_login set but user was not given" unless defined $user;
-                    croak "ask_for_username_at_login set can not be user with a custom password prompt"
+                    croak "ask_for_username_at_login can not be used with a custom password prompt"
                         if defined $password_prompt;
                 }
             }
         }
 
-        my $expect_log_user = delete $opts->{expect_log_user} || 0;
+        delete $opts->{expect_log_user}; # backward compatibility, not used anymore
 	my $stderr_discard = delete $opts->{stderr_discard};
 	my $stderr_fh = ($stderr_discard ? undef : delete $opts->{stderr_fh});
         my $open2_cmd = delete $opts->{open2_cmd};
@@ -271,7 +271,10 @@ sub _init_transport {
                 return;
             }
             $sftp->{pid} = $child;
-            $sftp->{_pty} = $pty;
+            open my $pty_dup, '+>&', $pty; # store pty as a file handler instead of a object in
+                                           # order to save it from being destroyed too early
+                                           # during global destruction
+            $sftp->{_pty} = $pty_dup;
 
             $debug and $debug & 65536 and _debug "starting password authentication";
             my $rv = '';
@@ -289,7 +292,7 @@ sub _init_transport {
                     }
                 }
 
-                if (waitpid($child, POSIX::WNOHANG()) > 0) {
+                if (waitpid($child, POSIX::WNOHANG()) > 0 or $! == Errno::ECHILD()) {
                     undef $sftp->{pid};
                     my $err = $? >> 8;
                     $sftp->_conn_failed("SSH slave exited unexpectedly with error code $err");
