@@ -1,11 +1,21 @@
 package Net::SFTP::Foreign::Common;
 
-our $VERSION = '1.65';
+our $VERSION = '1.76_02';
 
 use strict;
 use warnings;
 use Carp;
-use Scalar::Util qw(dualvar tainted);
+
+BEGIN {
+    # Some versions of Scalar::Util are crippled
+    require Scalar::Util;
+    eval { Scalar::Util->import(qw(dualvar tainted)); 1 }
+        or do {
+            *tainted = sub { croak "The version of Scalar::Util installed on your system "
+                                 . "does not provide 'tainted'" };
+            *dualvar = sub { $_[0] };
+        };
+}
 
 use Net::SFTP::Foreign::Helpers qw(_gen_wanted _ensure_list _debug _glob_to_regex _is_lnk _is_dir $debug);
 use Net::SFTP::Foreign::Constants qw(:status);
@@ -59,21 +69,32 @@ sub _set_error {
 	    $str = $code ? "Unknown error $code" : "OK";
 	}
         $debug and $debug & 64 and _debug("_set_err code: $code, str: $str");
-	return $sftp->{_error} = dualvar $code, $str;
+	my $error = $sftp->{_error} = dualvar $code, $str;
+
+        # FIXME: use a better approach to determine when some error is fatal
+        croak $error if $sftp->{_autodie};
     }
-    else {
-	return $sftp->{_error} = 0;
+    elsif ($sftp->{_error}) {
+        # FIXME: use a better approach to determine when some error is fatal
+        if ($sftp->{_error} != Net::SFTP::Foreign::Constants::SFTP_ERR_CONNECTION_BROKEN()) {
+            $sftp->{_error} = 0;
+        }
     }
+    return $sftp->{_error}
 }
 
 sub _clear_error_and_status {
     my $sftp = shift;
-    $sftp->{_error} = 0;
-    $sftp->{_status} = 0;
+    $sftp->_set_error;
+    $sftp->_set_status;
 }
 
 sub _copy_error {
-    $_[0]->{_error} = $_[1]->{_error};
+    my ($sftp, $other) = @_;
+    unless ($sftp->{_error} and
+            $sftp->{_error} == Net::SFTP::Foreign::Constants::SFTP_ERR_CONNECTION_BROKEN()) {
+        $sftp->{_error} = $other->{_error};
+    }
 }
 
 sub error { shift->{_error} }
@@ -83,6 +104,13 @@ sub die_on_error {
     $sftp->{_error} and croak(@_ ? "@_: $sftp->{_error}" : $sftp->{_error});
 }
 
+sub _ok_or_autodie {
+    my $sftp = shift;
+    return 1 unless $sftp->{_error};
+    $sftp->{_autodie} and croak $sftp->{_error};
+    undef;
+}
+
 sub _set_errno {
     my $sftp = shift;
     if ($sftp->{_error}) {
@@ -109,12 +137,20 @@ sub _set_errno {
     }
 }
 
+sub _best_effort {
+    my $sftp = shift;
+    my $best_effort = shift;
+    my $method = shift;
+    local ($sftp->{_error}, $sftp->{_autodie}) if $best_effort;
+    $sftp->$method(@_);
+    return (($best_effort or not $sftp->{_error}) ? 1 : undef);
+}
+
 sub _call_on_error {
     my ($sftp, $on_error, $entry) = @_;
     $on_error and $sftp->error
 	and $on_error->($sftp, $entry);
-    $sftp->_set_error;
-    $sftp->_set_status;
+    $sftp->_clear_error_and_status;
 }
 
 # this method code is a little convoluted because we are trying to
@@ -125,12 +161,12 @@ sub find {
     my $self = shift;
     my %opts = @_ & 1 ? ('dirs', @_) : @_;
 
-    $self->_set_error;
-    $self->_set_status;
+    $self->_clear_error_and_status;
 
     my $dirs = delete $opts{dirs};
     my $follow_links = delete $opts{follow_links};
     my $on_error = delete $opts{on_error};
+    local $self->{_autodie} if $on_error;
     my $realpath = delete $opts{realpath};
     my $ordered = delete $opts{ordered};
     my $names_only = delete $opts{names_only};
@@ -262,6 +298,7 @@ sub glob {
     return () if $glob eq '';
 
     my $on_error = delete $opts{on_error};
+    local $sftp->{_autodie} if $on_error;
     my $follow_links = delete $opts{follow_links};
     my $ignore_case = delete $opts{ignore_case};
     my $names_only = delete $opts{names_only};
@@ -276,10 +313,16 @@ sub glob {
 
     my $wantarray = wantarray;
 
-    my @parts = ($glob =~ m{\G/*([^/]+)}g);
-    push @parts, '.' unless @parts;
-
-    my $top = ($glob =~ m|^/|) ? '/' : '.';
+    my (@parts, $top);
+    if (ref $glob eq 'Regexp') {
+        @parts = ($glob);
+        $top = '.';
+    }
+    else {
+        @parts = ($glob =~ m{\G/*([^/]+)}g);
+        push @parts, '.' unless @parts;
+        $top = ( $glob =~ m|^/|  ? '/' : '.');
+    }
     my @res = ( {filename => $top} );
     my $res = 0;
 
@@ -287,7 +330,14 @@ sub glob {
 	my @parents = @res;
 	@res = ();
 	my $part = shift @parts;
-	my ($re, $has_wildcards) = _glob_to_regex($part, $strict_leading_dot, $ignore_case);
+        my ($re, $has_wildcards);
+        if (ref $part eq 'Regexp') {
+            $re = $part;
+            $has_wildcards = 1;
+        }
+	else {
+            ($re, $has_wildcards) = _glob_to_regex($part, $strict_leading_dot, $ignore_case);
+        }
 
 	for my $parent (@parents) {
 	    my $pfn = $parent->{filename};
@@ -304,7 +354,7 @@ sub glob {
                                            $e->{a} = $a;
                                        }
                                        else {
-                                           $sftp->_call_on_error($on_error, $e);
+                                           $on_error and $sftp->_call_on_error($on_error, $e);
                                            return undef;
                                        }
                                    }
@@ -316,7 +366,7 @@ sub glob {
                                            if ($realpath) {
                                                my $rp = $e->{realpath} = $sftp->realpath($e->{filename});
                                                unless (defined $rp) {
-                                                   $sftp->_call_on_error($on_error, $e);
+                                                   $on_error and $sftp->_call_on_error($on_error, $e);
                                                    return undef;
                                                }
                                            }
@@ -329,7 +379,7 @@ sub glob {
                                }
                                return undef
                            } )
-                    or $sftp->_call_on_error($on_error, $parent);
+                    or ($on_error and $sftp->_call_on_error($on_error, $parent));
             }
             else {
                 my $fn = $sftp->join($pfn, $part);
@@ -344,7 +394,7 @@ sub glob {
                             if ($realpath) {
                                 my $rp = $fn = $e->{realpath} = $sftp->realpath($fn);
                                 unless (defined $rp) {
-                                    $sftp->_call_on_error($on_error, $e);
+                                    $on_error and $sftp->_call_on_error($on_error, $e);
                                     next;
                                 }
                             }
@@ -361,8 +411,29 @@ sub glob {
 
 sub test_d {
     my ($sftp, $name) = @_;
-    my $a = $sftp->stat($name);
-    $a ? _is_dir($a->perm) : undef;
+    {
+        local $sftp->{_autodie};
+        my $a = $sftp->stat($name);
+        return _is_dir($a->perm) if $a;
+    }
+    if ($sftp->{_status} == SSH2_FX_NO_SUCH_FILE) {
+        $sftp->_clear_error_and_status;
+        return undef;
+    }
+    $sftp->_ok_or_autodie;
+}
+
+sub test_e {
+    my ($sftp, $name) = @_;
+    {
+        local $sftp->{_autodie};
+        $sftp->stat($name) and return 1;
+    }
+    if ($sftp->{_status} == SSH2_FX_NO_SUCH_FILE) {
+        $sftp->_clear_error_and_status;
+        return undef;
+    }
+    $sftp->_ok_or_autodie;
 }
 
 1;
