@@ -43,11 +43,12 @@ our %DEFAULTS = ( put => [best_effort => 1],
                   new => [] );
 
 BEGIN {
-    my @forbidden = qw( setcwd cwd open opendir sftpread sftpwrite seek
-                        tell eof write flush read getc lstat stat fstat
-                        remove rmdir mkdir setstat fsetstat close closedir
-                        readdir realpath readlink rename symlink abort
-                        get_content join glob rremove rget rput error );
+    my @forbidden = qw( setcwd cwd open opendir sftpread sftpwrite
+                        seek tell eof write flush read getc lstat stat
+                        fstat remove rmdir mkdir setstat fsetstat
+                        close closedir readdir realpath readlink
+                        rename symlink abort get_content join glob
+                        rremove rget rput error die_on_error );
 
     for my $method (@forbidden) {
         my $super = "SUPER::$method";
@@ -231,7 +232,7 @@ __END__
 
 =head1 NAME
 
-Net::SFTP::Foreign::Compat - Adaptor for Net::SFTP compatibility
+Net::SFTP::Foreign::Compat - Adapter for Net::SFTP compatibility
 
 =head1 SYNOPSIS
 
@@ -261,7 +262,7 @@ to Net::SFTP::Foreign.
 
 The hash C<%Net::SFTP::Foreign::DEFAULTS> can be used to set default
 values for L<Net::SFTP::Foreign> methods called under the hood and
-otherwise not accesible through the Net::SFTP API.
+otherwise not accessible through the Net::SFTP API.
 
 The entries currently supported are:
 
