@@ -17,9 +17,11 @@ flock DATA, Fcntl::LOCK_EX();
 sub is_windows { $^O =~ /MSWin32/i }
 
 sub sftp_server {
-
     my ($sscmd, @ssh, $ssname);
 
+    $sscmd = $ENV{NET_SFTP_FOREIGN_TESTING_SFTP_SERVER_PATH};
+    return $sscmd if defined $sscmd;
+
     if(is_windows) {
 	$ssname = 'sftp-server.exe';
 	my $pf;
@@ -60,6 +62,7 @@ sub sftp_server {
 	      File::Spec->catfile($up, 'lib', $ssname),
               File::Spec->catfile($up, 'lib64', $ssname),
 	      File::Spec->catfile($up, 'libexec', $ssname),
+	      File::Spec->catfile($up, 'libexec', 'openssh', $ssname),
 	      File::Spec->catfile($up, 'sbin', $ssname),
 	      File::Spec->catfile($up, 'lib', 'openssh', $ssname),
               File::Spec->catfile($up, 'lib64', 'openssh', $ssname),
@@ -132,6 +135,12 @@ sub new_args {
         my $ss_cmd = sftp_server;
         defined $ss_cmd or plan skip_all => 'sftp-server not found';
         push @diag, "sftp-server found at $ss_cmd";
+
+        if ($ENV{NET_SFTP_FOREIGN_TESTING_WITH_WINE}) {
+            push @diag, "running it with wine!";
+            $ss_cmd = [wine => $ss_cmd];
+        }
+
         push @args, open2_cmd => $ss_cmd;
     }
     diag join(", ", @diag) if @diag;
